@@ -2,16 +2,22 @@
 
 =head1 NAME 
 
-transord - translate a word from specified language.
+transword - translate a word from specified language.
 
 =head1 DESCRIPTION
 
 This is a little sample program which will use the dictionary build up
-by addword to translate words from the given language to the other (English to polish or the other way round).
+by addword to translate words from the given language to the other
+(English to polish or the other way round).
+
+=head1 FILES
+
+This uses a BiIndex consisting of two files in the current directory:
+english-polish and polish-english.
 
 =cut
 
-use BiIndex;
+use CDB_File::BiIndex;
 
 my $language=shift;
 
@@ -21,9 +27,9 @@ my $language=shift;
 
 
 if ($language =~ m/^e/i) {
-    $::index=new BiIndex "english-polish", "polish-english"; 
+    $::index=new CDB_File::BiIndex "english-polish", "polish-english";
 } elsif ($language =~ m/^p/i) {
-    $::index=new BiIndex "polish-english", "english-polish"; 
+    $::index=new CDB_File::BiIndex "polish-english", "english-polish";
 } else {
     die "You must choose a language, English or Polish\n";
 }
@@ -33,16 +39,10 @@ die "Give words to translate\n" unless @ARGV;
 foreach (@ARGV) {
     my $translations_list = $::index->lookup_first($_);
     if ($translations_list) {
-	print "$_ translates as:-\n";
-	my ($key, $value);
-	while ( ($key, $value)=each %{$translations_list} ) {
-	    print "\t$key\n";
-	}
+	print "$_ translates as:-\n  ";
+	print join ("\n  ", @$translations_list), "\n\n";
     } else {
 	print "$_ has no known translation\n";
     }
-} continue {
-    print "\n";
 }
 
-
